﻿using System.Linq;
using Rhino;
using Rhino.Commands;
using Rhino.DocObjects;
using Rhino.Geometry;
using Rhino.Input;

namespace FlowByCurve
{
    [System.Runtime.InteropServices.Guid("91d0a668-5ba2-4c90-92a9-9042a052d141")]
    public class FlowByCurveCommand : Command
    {
        public FlowByCurveCommand()
        {
            // Rhino only creates one instance of each command class defined in a
            // plug-in, so it is safe to store a refence in a static property.
            Instance = this;
        }

        ///<summary>The only instance of this command.</summary>
        public static FlowByCurveCommand Instance
        {
            get; private set;
        }

        ///<returns>The command name as it appears on the Rhino command line.</returns>
        public override string EnglishName => "FlowByCurveCommand";

        protected override Result RunCommand(RhinoDoc doc, RunMode mode)
        {
            ObjRef[] curvesToFlow;
            ObjRef objRefSourceCrv, objRefTargetCrv;

            //Get Curves
            if (RhinoGet.GetOneObject("Source curve", false, ObjectType.Curve, out objRefSourceCrv) != Result.Success) return Result.Failure;
            RhinoDoc.ActiveDoc.Objects.UnselectAll();
            if (RhinoGet.GetOneObject("Target curve", false, ObjectType.Curve, out objRefTargetCrv) != Result.Success) return Result.Failure;
            RhinoDoc.ActiveDoc.Objects.UnselectAll();
            if (RhinoGet.GetMultipleObjects("Curves To Flow", false, ObjectType.Curve, out curvesToFlow) != Result.Success) return Result.Failure;

            //Flow Curves
            var mc = new MorphControl(objRefSourceCrv.Curve().ToNurbsCurve(), objRefTargetCrv.Curve().ToNurbsCurve()) { PreserveStructure = true };
            var outputCrvs = curvesToFlow.Select(objRef => objRef.Curve()).ToList();
            outputCrvs.ForEach(_ => mc.Morph(_));

            //Add to Doc
            outputCrvs.ForEach(_=>RhinoDoc.ActiveDoc.Objects.AddCurve(_));

            return Result.Success;
        }
    }
}
